//*****************************************************************************
//*
//*
//*		EthernetSpi.c
//*
//*
//*****************************************************************************
#define 	LWIP_PROVIDE_NO_ERRNO
#include	"../lwipopts.h"
#include	"lwip/sys.h"
#include	"lwip/mem.h"
#include	"lwip/memp.h"
#include	"lwip/arch.h"
#include	"lwip/dhcp.h"
#include	"lwip/stats.h"
#include	"lwip/tcpip.h"
#include	"lwip/sockets.h"
#include	"netif/etharp.h"



#define 	THREAD_PRIORITY			0
#define		MAX_INTERFACES			4
#define 	ETHERSIZE  				1514



extern  void 	IpSetHostname(const char *pName);
extern	unsigned long __stdcall GetLastError(void);


struct netif		    aInterface [MAX_INTERFACES];
static VnetHandle		aNetHandles[MAX_INTERFACES];



//*****************************************************************************
//*
//*		EthSendProc
//*
//*****************************************************************************
//	Send function for outgoing network packets
err_t EthSendProc(struct netif *pInterface,struct pbuf *pBuffer)
{
unsigned char   aMem[ETHERSIZE+32];
struct pbuf    *pTemp;
unsigned char  *pPtr;
unsigned		uPos;
unsigned		uNum;
unsigned		uLen;
unsigned		uRet;
err_t			iErr;




	if(!pBuffer || !pInterface        )return ERR_ARG;	// check parameters
	if(!pInterface->num>MAX_INTERFACES)return ERR_IF;	// check interface number

	uNum = pInterface->num;
	iErr = ERR_OK;

	if(pBuffer->next)									// buffer chain
		{
		uPos = 0;
														// copy the parts into a singel buffer
		for(pTemp=pBuffer;pTemp!=NULL;pTemp=pTemp->next)
			{
			if(uPos+pTemp->len>ETHERSIZE)
				{
				return ERR_BUF;
				}

			memcpy(aMem+uPos,pTemp->payload,pTemp->len);
			uPos = uPos+pTemp->len;
			}

		uLen = uPos;
		pPtr = aMem;
		}
	else{												// single buffer
		if(pBuffer->len>ETHERSIZE)
			{
			return ERR_BUF;
			}

		uLen = pBuffer->len;							// the packet buffer directly
		pPtr = pBuffer->payload;
		}


	PIPE_STATS_INC(pipe.output);


	   uRet = VnetSend(aNetHandles[uNum],pPtr,uLen);	// check if packet is transmitted
	if(uRet)
		{
		ThreadSleep(1);

														// try it again after 1 ms
		   uRet = VnetSend(aNetHandles[uNum],pPtr,uLen);
		if(uRet)
			{
			ThreadSleep(5);

														// try it finaly after 5 ms
			   uRet = VnetSend(aNetHandles[uNum],pPtr,uLen);
			if(uRet)
				{
				PIPE_STATS_INC(pipe.outerr);
				iErr = ERR_TIMEOUT;
				}
			else{
				iErr = ERR_IF;
				}
			}
		}


return iErr;
}

//*****************************************************************************
//*
//*		EthRecvThread
//*
//*****************************************************************************
//	Receive thread for incoming network packets
static void __cdecl EthRecvThread(void *pParam)
{
struct netif       *pInterface=(struct netif*)pParam;
unsigned char		aData[1600];
struct pbuf		   *pBuffer;
int					iSize;
unsigned			uRecv;
unsigned			uNum;
unsigned			uErr;
unsigned			uOk;




	   uNum = pInterface->num;
	if(uNum>=MAX_INTERFACES)							// check interface number
		{
		return;
		}


	uRecv = 0;

	netif_set_up(pInterface);							// bring an interface up




	for(;;)
		{												// get next packet from VNET
		   iSize = VnetRecv(aNetHandles[uNum],aData,sizeof(aData));
		if(iSize<=0)continue;

		uRecv++;

		uOk = 1;
		PIPE_STATS_INC(pipe.input);
														// allocate a buffer for the reseived packet
		   pBuffer = pbuf_alloc(PBUF_RAW,iSize,PBUF_RAM);
		if(pBuffer)
			{
			memcpy(pBuffer->payload,aData,iSize);
			uErr = pInterface->input(pBuffer,pInterface);

			if(uErr!=ERR_OK)
				{
				ThreadSleep(5);
														// put the packet to the LWIP stack
				uErr = pInterface->input(pBuffer,pInterface);

				if(uErr!=ERR_OK)
					{
					pbuf_free(pBuffer);					// dealloc the packet buffer at an error

					PIPE_STATS_INC(pipe.inperr);
					uOk = 0;
					}
				}
			}
		}

	VnetClose(aNetHandles[uNum]);						// close VNET connection
	aNetHandles[uNum] = 0;

}

//*****************************************************************************
//*
//*		ArpTimer
//*
//*****************************************************************************
//	Callback function to clear expired ARB entries.
static void ArpTimer(void *pParaam)
{

	etharp_tmr();										// clear entries
	sys_timeout(ARP_TMR_INTERVAL,ArpTimer,NULL);		// register cb again

}

//*****************************************************************************
//*
//*		NetInterfaceInitCb
//*
//*****************************************************************************
//	Callback for netif_add(...), here the VNET interface will be activated
static err_t NetInterfaceInitCb(struct netif *pInterface)
{
unsigned	uNum;


	uNum = pInterface->num;

	LWIP_ASSERT("pInterface != NULL", (pInterface!=NULL));
	LWIP_ASSERT("pInterface->num   ", (uNum<MAX_INTERFACES));

														// open a VNET port
		aNetHandles[uNum] = VnetOpen(0,VNET_AUTOPORT,NULL,200,1,0);
	if(!aNetHandles[uNum])
		{
		return ERR_IF;
		}

														// use MAC address from VNET port
	memcpy(pInterface->hwaddr,VnetMac(aNetHandles[uNum]),6);


	#if LWIP_NETIF_HOSTNAME
		pInterface->hostname =  IpGetHostname();		// initialize interface hostname
	if(*pInterface->hostname==0)IpSetHostname("lwip");	// see at SocketUtility.cpp
	#endif

	pInterface->name[0]		= 'e';
	pInterface->name[1]		= '0'+uNum;
	pInterface->flags	   |= NETIF_FLAG_ETHARP;
	pInterface->flags	   |= NETIF_FLAG_LINK_UP;
	pInterface->flags	   |= NETIF_FLAG_BROADCAST;
	pInterface->output		= etharp_output;
	pInterface->linkoutput	= EthSendProc;


	etharp_init();
	sys_timeout(ARP_TMR_INTERVAL,ArpTimer,NULL);		// start the arp timer

														// start the packet receive thread
	ThreadStartEx(EthRecvThread,0x8000,pInterface,THREAD_PRIORITY,"EthRecvThread");



return ERR_OK;
}


//*****************************************************************************
//*
//*		InitLwIp
//*
//*****************************************************************************
//	Initalizes all LWIP network interfaces
int InitLwIp()
{
struct ip_addr	sGateWay;
struct ip_addr	sNetMask;
struct ip_addr	sIpAddr;


	tcpip_init(NULL,NULL);								// create tcp_ip stack thread


	IP4_ADDR(&sIpAddr ,192,168, 99, 33);
	IP4_ADDR(&sGateWay,192,168, 99,  1);
	IP4_ADDR(&sNetMask,255,255,255,  0);

														// start the first interface
	if(!netif_add(aInterface+0,&sIpAddr,&sNetMask,&sGateWay,NULL,&NetInterfaceInitCb,&tcpip_input))
		{
		return -1;
		}


//	IP4_ADDR(&sIpAddr ,192,168, 98, 33);
//	IP4_ADDR(&sGateWay,192,168, 98,  1);
//	IP4_ADDR(&sNetMask,255,255,255,  0);
//
//														// start the next interface
//	if(!netif_add(aInterface+1,&sIpAddr,&sNetMask,&sGateWay,NULL,&NetInterfaceInitCb,&tcpip_input))
//		{
//		return -1;
//		}



return 0;
}
