//*****************************************************************************
//*
//*
//*		sys_arch_win.c
//*
//*
//*****************************************************************************
#include	<string.h>
#include	"lwip/opt.h"
#include	"lwip/sys.h"
#include	"lwip/tcp.h"
#include	"lwip/tcpip.h"
#include	"lwip/api.h"
#include	"lwip/api_msg.h"
#include	"lwip/stats.h"
#include	"netif/etharp.h"

#if			SYS_ARCH_PRIOR_FILLED
static int *filled=0;
int 		sys_get_filled(){return (filled)? *filled:0;}
#endif

ThreadMutex			hLwIpMux;

extern unsigned		PriorityOfPort(unsigned short uLocalPort,unsigned short uRemotePort,int iType);

#if		LWIP_PLATFORM_BYTESWAP==0
#ifndef	LWIP_PLATFORM_HTONS
#define	LWIP_PLATFORM_HTONS(x)		((((x)&0x00FF)<<8) | (((x)&0xFF00)>>8))
#endif
#endif

//*****************************************************************************
//*
//*		netconn_set_priority
//*
//*****************************************************************************
//	set the priority for a net tcp connection, is used in the pipe
#if TCP_PRIORITIES>1
void netconn_set_priority(struct netconn *conn)
{

	if(conn && conn->type==NETCONN_TCP && conn->pcb.tcp && conn->pcb.tcp->state!=LISTEN)
		{
		conn->priority = PriorityOfPort(conn->pcb.tcp->local_port,conn->pcb.tcp->remote_port,(conn->flags&NETCONN_FLAG_WAS_ACCEPTED)? 1:0);
		}


}
#endif

//*****************************************************************************
//*
//*		sys_init
//*
//*****************************************************************************
void sys_init(void)
{

	if(!hLwIpMux)
		{
		hLwIpMux  = ThreadMutexInit();
		}


	#ifdef LWIP_DEBUG

	if(!hPrintMux)
		{
		hPrintMux = ThreadMutexInit();
		}

	#endif
}


//*****************************************************************************
//*
//*		sys_arch_mbox_new
//*
//*****************************************************************************
//  Creates an empty mailbox.
int sys_arch_mbox_new(sys_mbox_t *pMbox,int iSize)
{
Mailslot		hMbox;


		hMbox = MailslotCreate(sizeof(void*),iSize);
	if(!hMbox)
		{
		#if SYS_STATS
		lwip_stats.sys.mbox.err++;
		#endif

		return ERR_MEM;
		}

	#if SYS_STATS

	lwip_stats.sys.mbox.used++;

	if (lwip_stats.sys.mbox.max<lwip_stats.sys.mbox.used)
		{
        lwip_stats.sys.mbox.max = lwip_stats.sys.mbox.used;
		}

	#endif

	*pMbox = hMbox;


return ERR_OK;
}

//*****************************************************************************
//*
//*		sys_arch_mbox_free
//*
//*****************************************************************************
//	deallocates a mail box
//	mbox		: handle of the mail box
void sys_arch_mbox_free(sys_mbox_t hMbox)
{
unsigned char	aData[sizeof(void**)];


	if(MailslotGetWait(hMbox,aData,0)>0)
		{
		#if SYS_STATS
	    lwip_stats.sys.mbox.err++;
		#endif
		}

	MailslotDelete(hMbox);

	#if SYS_STATS
	lwip_stats.sys.mbox.used--;
	#endif

}

//*****************************************************************************
//*
//*		sys_arch_sem_new
//*
//*****************************************************************************
//  Creates and returns a new semaphore.
//	count	: is the initial state of the semaphore
//	Returns the handle of the semathore or 0 if an error occurs
int sys_arch_sem_new(sys_sem_t *pSem,unsigned uCount)
{
ThreadSemaphore		hSem;

		hSem = ThreadSemaphoreInitEx(uCount);
	if(!hSem)
		{
		#if SYS_STATS
		lwip_stats.sys.sem.err++;
		#endif

		*pSem  = SYS_SEM_NULL;

		return ERR_MEM;
		}


	#if SYS_STATS
	lwip_stats.sys.sem.used++;

 	if(lwip_stats.sys.sem.max<lwip_stats.sys.sem.used)
		{
		lwip_stats.sys.sem.max = lwip_stats.sys.sem.used;
		}

	#endif

	*pSem = hSem;


return ERR_OK;
}

//*****************************************************************************
//*
//*		sys_arch_sem_free
//*
//*****************************************************************************
//	Deallocates a semaphore
void sys_arch_sem_free(sys_sem_t hSem)
{

	ThreadSemaphoreExit(hSem);

	#if SYS_STATS
    lwip_stats.sys.sem.used--;
	#endif

}

//*****************************************************************************
//*
//*		sys_arch_pipe_check
//*
//*****************************************************************************
#ifndef _DEBUG
#define sys_arch_pipe_check(pBox)
#else
void sys_arch_pipe_check(sys_pipe_t *pBox)
{
struct tcpip_msg   *pNext;
unsigned			uCount;
unsigned			uPos;



	SYS_CHECK_POINTER(pBox,sizeof(sys_pipe_t))

	uCount = 0;

	for(uPos=0;uPos<TCP_PRIORITIES;uPos++)
		{
		if(!pBox->first[uPos] && !pBox->last[uPos])
			{
			continue;
			}

		assert(pBox->first[uPos] && pBox->last[uPos]);

		pNext = pBox->last[uPos];

		for(;;)
			{
			SYS_CHECK_POINTER(pNext,sizeof(*pNext));

			uCount++;

			if(!pNext->next)break;

			pNext = pNext->next;
			}
		}

	#if	SYS_ARCH_PRIOR_FILLED
	assert((pBox->filled^uCount)==0);
	#endif

}
#endif

//*****************************************************************************
//*
//*		sys_arch_pipe_new
//*
//*****************************************************************************
//  Creates an empty priority mailbox.
int sys_arch_pipe_new(sys_pipe_t *pBox,int iSize)
{
unsigned	uPos=0;
unsigned	uErr=0;



	   uErr = sys_arch_sem_new(&pBox->sem,0);
	if(uErr!=ERR_OK)
		{
		return uErr;
		}

	   uErr = sys_sem_new(&pBox->lock,1);
	if(uErr != ERR_OK)
		{
		sys_arch_sem_free(&pBox->sem);
		return uErr;
		}


	#if TCP_PRIORITIES>1

	for(uPos=0;uPos<TCP_PRIORITIES;uPos++)
		{
		pBox->first[uPos] = 0;
		pBox->last [uPos] = 0;
		}

	pBox->state		= 0;

	#else

	pBox->first[0] = 0;
	pBox->last [0] = 0;

	#endif

	#if	SYS_ARCH_PRIOR_FILLED
	pBox->filled = 0;
	filled = &pBox->filled;
	#endif

	sys_arch_pipe_check(pBox);


return ERR_OK;
}

//*****************************************************************************
//*
//*		sys_arch_pipe_fetch
//*
//*****************************************************************************
unsigned sys_arch_pipe_fetch(sys_pipe_t *pBox,struct tcpip_msg **pMsg,unsigned uTimeout)
{
struct tcpip_msg   *pNext;
unsigned			uState;
unsigned			uPos;
unsigned			uNum;
unsigned			uRet;


	if(sys_arch_sem_wait(&pBox->sem,uTimeout))
		{
		*pMsg = 0;
		return SYS_ARCH_TIMEOUT;
		}

	SYS_CHECK_POINTER(pMsg,sizeof(*pMsg))

	sys_arch_sem_wait(&pBox->lock,0);

	sys_arch_pipe_check(pBox);

	#if TCP_PRIORITIES>1

	uState = pBox->state;
	pBox->state++;


	uNum   = uState%(1<<TCP_PRIORITIES);

	for(uPos=0;uNum;uPos++,uNum>>=1)
		{
		if(uNum&1)break;
		}


	for(uNum=0;;uNum++)
		{
		if(uNum>=TCP_PRIORITIES)
			{
			uRet = SYS_ARCH_TIMEOUT;
		   *pMsg = 0;

			#if SYS_STATS
			lwip_stats.sys.mbox.err++;
			#endif

			break;
			}

		if(!pBox->last[uPos])
			{
			   uPos++;
			if(uPos>=TCP_PRIORITIES)uPos = 0;
			continue;
			}

		#if	SYS_ARCH_PRIOR_FILLED
		LockSub(&pBox->filled,1);
		#endif

		SYS_CHECK_POINTER(&pBox->last[uPos]      ,sizeof(pBox->last[uPos]      ))
		SYS_CHECK_POINTER(&pBox->last[uPos]->next,sizeof(pBox->last[uPos]->next))

		pNext			 = pBox->last[uPos];
	   *pMsg			 = pNext;
	    pNext			 = pNext->next;
		pBox->last[uPos] = pNext;

		if(!pNext)pBox->first[uPos]=0;

		uRet = 0;
		break;
		}

	#else

	if(pBox->last[0]==0)
		{
		uRet 	= SYS_ARCH_TIMEOUT;
	   *pMsg 	= 0;
		uPos	= 0;
		uNum	= 0;
		uState	= 0;

		#if SYS_STATS
		lwip_stats.sys.mbox.err++;
		#endif
		}
	else{
		#if	SYS_ARCH_PRIOR_FILLED
		LockSub(&pBox->filled,1);
		#endif

		SYS_CHECK_POINTER(&pBox->last[0]      ,sizeof(pBox->last[0]      ))
		SYS_CHECK_POINTER(&pBox->last[0]->next,sizeof(pBox->last[0]->next))

		pNext			 = pBox->last[0];
	   *pMsg			 = pNext;
		pNext			 = pNext->next;
		pBox->last[0]    = pNext;

		if(!pNext)pBox->first[0]=0;

		uRet = 0;
		}

#endif

	sys_arch_pipe_check(pBox);

	sys_sem_signal(&pBox->lock);



return uRet;
}

//*****************************************************************************
//*
//*		sys_arch_pipe_post
//*
//*****************************************************************************
void sys_arch_pipe_post(sys_pipe_t *pBox,struct tcpip_msg *pMsg)
{

#if TCP_PRIORITIES>1

	struct tcpip_msg   *pTcpMsg=(struct tcpip_msg*)pMsg;
	unsigned char	   *pPacket;
	unsigned short 		uPortD;
	unsigned short 		uPortS;
	unsigned			uPrior;
	unsigned			uSize;
	unsigned			uLen;



	SYS_CHECK_POINTER(pMsg,sizeof(*pMsg))

	if(pTcpMsg->type==TCPIP_MSG_API)					// api input
		{
		uPrior	= pTcpMsg->msg.apimsg->msg.conn->priority;
		}
	else if(pTcpMsg->type==TCPIP_MSG_INPKT)				// received packet
		{
		pPacket	= (unsigned char*)(pTcpMsg->msg.inp.p->payload);
		uSize	= pTcpMsg->msg.inp.p->len;
		uPrior	= TCP_DEFAULT_PRIOR;

		if(uSize>14+20+8 && *(u16_t*)(pPacket+12)==PP_HTONS(ETHTYPE_IP))
			{											// tcp or udp
			if(pPacket[23]==IP_PROTO_TCP || pPacket[23]==IP_PROTO_UDP)
				{
				pPacket += 14;
				uLen	 = (pPacket[0]&0x0F)*4;
				uPortS	 = LWIP_PLATFORM_HTONS(*(u16_t*)(pPacket+uLen  ));
				uPortD	 = LWIP_PLATFORM_HTONS(*(u16_t*)(pPacket+uLen+2));
				uPrior	 = PriorityOfPort(uPortD,uPortS,2);
				}
			}
		}
	else{												// other messages
		uPrior	= TCP_DEFAULT_PRIOR;
		}

	if(uPrior>=TCP_PRIORITIES)
		{
		uPrior	= TCP_PRIORITIES-1;
		}


	sys_arch_sem_wait(&pBox->lock,0);

	sys_arch_pipe_check(pBox);

	pMsg->next = 0;



	if(pBox->first[uPrior])
		{
		SYS_CHECK_POINTER(&pBox->first[uPrior]->next,sizeof(pBox->first[uPrior]->next))
		SYS_CHECK_POINTER(&pBox->first[uPrior]      ,sizeof(pBox->first[uPrior]      ))
		SYS_CHECK_POINTER(&pBox->last [uPrior]      ,sizeof(pBox->last [uPrior]      ))

		pBox->first[uPrior]->next	= pMsg;
		pBox->first[uPrior]			= pMsg;
		}
	else{
		SYS_CHECK_POINTER(&pBox->first[uPrior]      ,sizeof(pBox->first[uPrior]      ))
		SYS_CHECK_POINTER(&pBox->last [uPrior]      ,sizeof(pBox->last [uPrior]      ))

		pBox->first[uPrior]			= pMsg;
		pBox->last [uPrior]			= pMsg;
		}

	#if	SYS_ARCH_PRIOR_FILLED
	LockAdd(&pBox->filled,1);
	#endif

	sys_arch_pipe_check(pBox);

	sys_sem_signal(&pBox->lock);

#else


	sys_arch_wait(&pBox->lock);

	sys_arch_pipe_check(pBox);

	pMsg->next = 0;

	if(pBox->first[0])
		{
		SYS_CHECK_POINTER(&pBox->first[0]->next,sizeof(pBox->first[0]->next))
		SYS_CHECK_POINTER(&pBox->first[0]      ,sizeof(pBox->first[0]      ))
		SYS_CHECK_POINTER(&pBox->last [0]      ,sizeof(pBox->last [0]      ))

		pBox->first[0]->next	= pMsg;
		pBox->first[0]			= pMsg;
		}
	else{
		SYS_CHECK_POINTER(&pBox->first[0]      ,sizeof(pBox->first[0]      ))
		SYS_CHECK_POINTER(&pBox->last [0]      ,sizeof(pBox->last [0]      ))

		pBox->first[0]			= pMsg;
		pBox->last [0]			= pMsg;
		}

	#if	SYS_ARCH_PRIOR_FILLED
	LockAdd(&pBox->filled,1);
	#endif

	sys_arch_pipe_check(pBox);

	sys_sem_signal(&pBox->lock);


#endif


	ThreadSemaphoreRelease(pBox->sem);

}


//*****************************************************************************
//*
//*		Logging Functions
//*
//*****************************************************************************

#if SYS_ARCH_DO_LOG

void **get_socket(int s);

struct netconn *netconn_of_port(unsigned short src,unsigned short dest)
{
static unsigned char	 hash[0x10000];
struct netconn			*conn;
void				   **sock;
unsigned				 s;


	src  = htons(src );
	dest = htons(dest);
	s    = hash[(src==TTE_PORT)? dest:src];

	   sock = get_socket(s);
	if(sock)
		{
		   conn = (struct netconn*)*sock;
		if(conn)
			{
			if(conn->type==NETCONN_TCP)
			if(conn->pcb.tcp!=0)
			if(conn->pcb.tcp->state!=LISTEN)

			if((conn->pcb.tcp->remote_port==dest && conn->pcb.tcp->remote_port==src) ||
			   (conn->pcb.tcp->local_port ==dest && conn->pcb.tcp->local_port ==src))
				{
				return conn;
				}
			}
		}

	for(s=0;s<MEMP_NUM_NETCONN;s++)
		{
			sock = get_socket(s);
		if(!sock)continue;

		conn = (struct netconn*)*sock;

		if(!conn)continue;

		if(conn->type!=NETCONN_TCP)continue;
	    if(conn->pcb.tcp==0)continue;
		if(conn->pcb.tcp->state==LISTEN)continue;

		if(conn->pcb.tcp->remote_port!=dest && conn->pcb.tcp->remote_port!=src)
		if(conn->pcb.tcp->local_port !=dest && conn->pcb.tcp->local_port !=src)continue;

		hash[(src==TTE_PORT)? dest:src] = s;
		return conn;
		}


return 0;
}

TteInfo *TteOfConn(TteInfo *info,struct netconn *conn,const char *name)
{

	info->pName			= name;
	info->iMemCount		= lwip_stats.memp[10].used;
	#if	SYS_ARCH_PRIOR_FILLED
	info->iMboxCount	= (filled)? *filled:-1;
	#endif

	if(conn && conn->type!=NETCONN_TCP)
		{
		return 0;
		}

	if(conn && conn->type==NETCONN_TCP && conn->pcb.tcp && conn->pcb.tcp->state!=LISTEN)
		{
		info->usPort		=(conn->pcb.tcp->local_port==TTE_PORT)? conn->pcb.tcp->remote_port:conn->pcb.tcp->local_port;
		info->usConnFlags	= conn->flags;
		info->usConnState	= conn->state;
		info->iRecvAvail	= conn->recv_avail;
		}

return info;
}

TteInfo *TteOfSock(TteInfo *info,int socket,const char *name)
{
struct netconn	*conn;
void		   **sock;


		sock = get_socket(socket);
	if(!sock)
		{
		info->pName			= name;
		#if	SYS_ARCH_PRIOR_FILLED
		info->iMboxCount	= (filled)? *filled:-1;
		#endif
		return info;
		}

	conn = (struct netconn*)*sock;

return TteOfConn(info,conn,name);
}

TteInfo *TteOfPort(TteInfo *info,unsigned short src,unsigned short dest,const char *name)
{
struct netconn	*conn;

	conn = netconn_of_port(src,dest);

return TteOfConn(info,conn,name);
}

TteInfo *TteOfNull(TteInfo *info,const char *name)
{

	info->pName			= name;
	#if	SYS_ARCH_PRIOR_FILLED
	info->iMboxCount	= (filled)? *filled:-1;
	#endif


return info;
}

TteInfo *TteOfData(TteInfo *info,const void *packet,const char *name)
{
unsigned char *data=(unsigned char *)packet;
unsigned	uPortS;
unsigned	uPortD;
unsigned	uLen;
unsigned	uAll;


	if(data && data[12]!=0x08 && data[13]!=0x00 && data[23]==0x06)
		{
		data			   +=  14;
		uLen				= (data[0]&0x0F)*4;
		uAll				=  data[2]*256+data[3];

		uPortS				=  data[uLen+0]*256+data[uLen+1];
		uPortD				=  data[uLen+2]*256+data[uLen+3];

		return TteOfPort(info,uPortS,uPortD,name);
		}

	info->pName			= name;
	#if	SYS_ARCH_PRIOR_FILLED
	info->iMboxCount	= (filled)? *filled:-1;
	#endif

return info;
}

#endif
