//*****************************************************************************
//*
//*
//*		sys_arch_free.h
//*
//*
//*****************************************************************************
#ifndef		__SYS_ATCH_H__
#define		__SYS_ATCH_H__


#include	"System/ThreadUtility.h"
#include	"System/Mailslots.h"
#include  	"FreeRTOS.h"
#include	<string.h>
#include	<assert.h>

#ifdef		__ICCARM__										// IAR Compiler
#pragma 	diag_suppress=Pa050
#pragma 	diag_suppress=Pe009
#endif

#if			SYS_STATS
#include	"lwip/stats.h"
#endif

#define		PERF_START
#define		PERF_STOP(x)

#define 	LWIP_PLATFORM_BYTESWAP 			0

#ifndef		SYS_ARCH_DO_LOG
#define 	SYS_ARCH_DO_LOG					0
#endif

#ifndef		SYS_ARCH_PRIOR_FILLED
#define 	SYS_ARCH_PRIOR_FILLED			SYS_ARCH_DO_LOG
#endif

#ifdef		SYS_ARCH_PRIOR_FILLED
extern		int sys_get_filled();
#else
#define 	sys_get_filled()				0
#endif

#define 	SYS_SEM_NULL					0
#define 	SYS_MBOX_NULL					0
#define 	SYS_ARCH_TIMEOUT				0xFFFFFFFF

#define		SYS_ARCH_ADD(var,val)			LockAdd(&var,val)
#define		SYS_ARCH_DEC(var,val)			LockSub(&var,val)

#define		SYS_ARCH_PROTECT(lev)			sys_arch_protect()
#define		SYS_ARCH_UNPROTECT(lev)			sys_arch_unprotect()
#define		SYS_ARCH_DECL_PROTECT(lev)

#define 	SYS_MSG_EXTENSION				struct tcpip_msg  *next;

#define 	SYS_CHECK_POINTER(p,s)


//*****************************************************************************
//*
//*		Inline Funktionen
//*
//*****************************************************************************

static inline void LockAdd(volatile int *pPtr,int iAdd)
{

	portDISABLE_INTERRUPTS();
	*((unsigned long*)pPtr) += iAdd;
	portENABLE_INTERRUPTS();
}

static inline void LockSub(volatile int *pPtr,int iAdd)
{

	portDISABLE_INTERRUPTS();
	*((unsigned long*)pPtr) -= iAdd;
	portENABLE_INTERRUPTS();

}

//*****************************************************************************
//*
//*		Definitionen und Funktionen
//*
//*****************************************************************************

typedef		void		   *sys_thread_t;
typedef		Mailslot		sys_mbox_t;
typedef		ThreadSemaphore	sys_sem_t;

extern		ThreadMutex		hLwIpMux;

#define		sys_arch_pool_map(memp)
#define 	sys_arch_sleep(ms)				ThreadSleep(ms)
#define 	sys_arch_jiffies()				ThreadTickCount()
#define 	sys_arch_now()					ThreadTickCount()

#define 	sys_arch_protect()				ThreadMutexWait(hLwIpMux)
#define 	sys_arch_unprotect()			ThreadMutexRelease(hLwIpMux)
#define 	sys_arch_thread_new(n,f,a,s,p)	ThreadStartEx(f,s,a,p,n)

#define		sys_arch_sem_signal(sem)		ThreadSemaphoreRelease(*sem)
#define		sys_arch_sem_valid(sem)			(*(sem)!=SYS_SEM_NULL)
#define		sys_arch_sem_set_invalid(sem)	 *(sem) =SYS_SEM_NULL

void		sys_arch_sem_free(sys_sem_t     hSem);
int 		sys_arch_sem_new (sys_sem_t    *pSem,unsigned uCount);

#define 	sys_arch_mbox_valid(mbox)		(*(mbox)!=SYS_MBOX_NULL)
#define		sys_arch_mbox_set_invalid(mbox)	 *(mbox) =SYS_MBOX_NULL

void		sys_arch_mbox_free(sys_mbox_t   hMbox);
int			sys_arch_mbox_new (sys_mbox_t  *pMbox,int iSize);

typedef	struct
	{
	sys_sem_t			lock;
	sys_sem_t			sem;
	#if TCP_PRIORITIES>1
	unsigned			state;
	struct tcpip_msg   *first[TCP_PRIORITIES];
	struct tcpip_msg   *last [TCP_PRIORITIES];
	#else
	struct tcpip_msg   *first[1];
	struct tcpip_msg   *last [1];
	#endif
	#if	SYS_ARCH_PRIOR_FILLED
	int					filled;
	#endif
	}sys_pipe_t;


#define 	sys_arch_pipe_valid(box)			((box)->sem!=SYS_SEM_NULL)
#define		sys_arch_pipe_set_invalid(box)		 (box)->sem =SYS_SEM_NULL
#define		sys_arch_pipe_free(box)				sys_arch_pipe_free_(&box)
#define		sys_arch_pipe_tryfetch(box,msg)		sys_arch_pipe_fetch(box,msg,0)

unsigned	sys_arch_pipe_fetch   (sys_pipe_t *pBox,struct tcpip_msg **pMsg,unsigned uTimeout);
void		sys_arch_pipe_post    (sys_pipe_t *pBox,struct tcpip_msg  *pMsg);
int			sys_arch_pipe_new     (sys_pipe_t *pBox,int iSize);


//*****************************************************************************
//*
//*		sys_arch_pipe_trypost
//*
//*****************************************************************************
static inline unsigned sys_arch_pipe_trypost(sys_pipe_t *pBox,struct tcpip_msg *pMsg)
{

	sys_arch_pipe_post(pBox,pMsg);

return 0;
}

//*****************************************************************************
//*
//*		sys_arch_sem_wait
//*
//*****************************************************************************
static inline unsigned sys_arch_sem_wait(sys_sem_t *pSem,unsigned uTimeout)
{

	if(!uTimeout)
		{
		ThreadSemaphoreWait(*pSem);
		return 0;
		}

	if(ThreadSemaphoreTimed(*pSem,uTimeout))
		{
		return 0;
		}


return SYS_ARCH_TIMEOUT;
}

//*****************************************************************************
//*
//*		sys_arch_mbox_fetch
//*
//*****************************************************************************
static inline unsigned sys_arch_mbox_fetch(sys_mbox_t *pMbox,void **pMsg,unsigned uTimeout)
{

	if(MailslotGetWait(*pMbox,pMsg,(uTimeout)? uTimeout:0xFFFFFFFF))
		{
		return 0;
		}

return SYS_ARCH_TIMEOUT;
}

//*****************************************************************************
//*
//*		sys_arch_mbox_tryfetch
//*
//*****************************************************************************
static inline unsigned sys_arch_mbox_tryfetch(sys_mbox_t *pMbox,void **pMsg)
{

	if(MailslotGetWait(*pMbox,pMsg,0))
		{
		return 0;
		}

return SYS_ARCH_TIMEOUT;
}

//*****************************************************************************
//*
//*		sys_arch_mbox_trypost
//*
//*****************************************************************************
static inline int sys_arch_mbox_trypost(sys_mbox_t *pMbox,void *pMsg)
{

	if(MailslotPut(*pMbox,&pMsg,sizeof(void*)))
		{
		return 0;
		}

	#if SYS_STATS
	lwip_stats.sys.mbox.err++;
	#endif

return -1;
}

//*****************************************************************************
//*
//*		sys_arch_mbox_post
//*
//*****************************************************************************
static inline void sys_arch_mbox_post(sys_mbox_t *pMbox,void *pMsg)
{

	MailslotPutWait(*pMbox,&pMsg,sizeof(void*),0xFFFFFFFF);

}


//*****************************************************************************
//*
//*		LOG functions
//*
//*****************************************************************************
#if SYS_ARCH_DO_LOG

#error 		"Not supported on FreeRtos"

#else

#define 	SYS_LOG_DATA(name,type,ptr,size,p1,p2,p3)
#define 	SYS_LOG_PORT(name,type,base,portl,portr,ptr,size,p1,p2,p3)
#define 	SYS_LOG_CONN(name,conn,p1,p2,p3)
#define 	SYS_LOG_SOCK(name,sock,p1,p2,p3)
#define 	SYS_LOG_NULL(name,p1,p2,p3)

#endif

#endif



