//*****************************************************************************
//*
//*
//*		WinEmulation.c
//*
//*
//*****************************************************************************
#include	"../../Tools/VirtualNetwork/VirtualNetworkTools.h"
#include	"System/ThreadUtility.h"
#include	<string.h>
#include	<stdlib.h>
#include	<stdio.h>

#if			SYS_ARCH_DO_LOG
#include	"../Tools/TcpTester.h"
#endif

#define 	LWIP_PROVIDE_NO_ERRNO
#include	"../lwipopts.h"
#include	"lwip/sys.h"
#include	"lwip/mem.h"
#include	"lwip/memp.h"
#include	"lwip/arch.h"
#include	"lwip/dhcp.h"
#include	"lwip/stats.h"
#include	"lwip/tcpip.h"
#include	"lwip/sockets.h"
#include	"netif/etharp.h"



#define 	THREAD_PRIORITY			0
#define		MAX_INTERFACES			4
#define 	PRINTING_ON				0
#define 	LOGGING_ON				0
#define 	ETHERSIZE  				1514



extern  void 	IpSetHostname(const char *pName);
extern	unsigned long __stdcall GetLastError(void);


struct netif		    aInterface [MAX_INTERFACES];
static VnetHandle		aNetHandles[MAX_INTERFACES];



//*****************************************************************************
//*
//*		EthSendProc
//*
//*****************************************************************************
//	Send function for outgoing network packets
err_t EthSendProc(struct netif *pInterface,struct pbuf *pBuffer)
{
unsigned char   aMem[ETHERSIZE+32];
struct pbuf    *pTemp;
unsigned char  *pPtr;
unsigned		uPos;
unsigned		uNum;
unsigned		uLen;
unsigned		uRet;
err_t			iErr;


	#if SYS_ARCH_DO_LOG
	TteInfo sInfo;
	#endif

	if(!pBuffer || !pInterface        )return ERR_ARG;	// check parameters
	if(!pInterface->num>MAX_INTERFACES)return ERR_IF;	// check interface number



	uNum = pInterface->num;
	iErr = ERR_OK;

	if(pBuffer->next)									// buffer chain
		{
		uPos = 0;
														// copy the parts into a singel buffer
		for(pTemp=pBuffer;pTemp!=NULL;pTemp=pTemp->next)
			{
			if(uPos+pTemp->len>ETHERSIZE)
				{
				return ERR_BUF;
				}

			memcpy(aMem+uPos,pTemp->payload,pTemp->len);
			uPos = uPos+pTemp->len;
			}

		uLen = uPos;
		pPtr = aMem;
		}
	else{												// single buffer
		if(pBuffer->len>ETHERSIZE)
			{
			return ERR_BUF;
			}

		uLen = pBuffer->len;							// the packet buffer directly
		pPtr = pBuffer->payload;
		}

	#if LOGGING_ON
	printf("send<--- %i    \t\t%05u\n",uLen,GetTickCount()%100000);
	#endif

	#if SYS_ARCH_DO_LOG
	memset(&sInfo,0,sizeof(sInfo));
	sInfo.pName		= "send";
	sInfo.iParam1	= uLen;
	sInfo.iMboxCount= sys_get_filled();
	TcpTesterPacket(&sInfo,pPtr,uLen,2);
	#endif

	PIPE_STATS_INC(pipe.output);


	   uRet = VnetSend(aNetHandles[uNum],pPtr,uLen);	// check if packet is transmitted
	if(uRet)
		{
		ThreadSleep(1);

		#if SYS_ARCH_DO_LOG
		memset(&sInfo,0,sizeof(sInfo));
		sInfo.pName		= "send 2nd";
		sInfo.iParam1	= uLen;
		sInfo.iParam2	= uRet;
		sInfo.iParam3	= GetLastError();
		sInfo.iMboxCount= sys_get_filled();
		TcpTesterPacket(&sInfo,pPtr,uLen,2);
		#endif
														// try it again after 1 ms
		   uRet = VnetSend(aNetHandles[uNum],pPtr,uLen);
		if(uRet)
			{
			ThreadSleep(5);

			#if SYS_ARCH_DO_LOG
			memset(&sInfo,0,sizeof(sInfo));
			sInfo.pName		= "send 3rd";
			sInfo.iParam1	= uLen;
			sInfo.iParam2	= uRet;
			sInfo.iParam3	= GetLastError();
			sInfo.iMboxCount= sys_get_filled();
			TcpTesterPacket(&sInfo,pPtr,uLen,2);
			#endif
														// try it finaly after 5 ms
			   uRet = VnetSend(aNetHandles[uNum],pPtr,uLen);
			if(uRet)
				{
				PIPE_STATS_INC(pipe.outerr);
				iErr = ERR_TIMEOUT;

				#if SYS_ARCH_DO_LOG
				memset(&sInfo,0,sizeof(sInfo));
				sInfo.pName		= "send error";
				sInfo.iParam1	= uLen;
				sInfo.iParam2	= uRet;
				sInfo.iParam3	= GetLastError();
				sInfo.iMboxCount= sys_get_filled();
				TcpTesterPacket(&sInfo,pPtr,uLen,0);
				#endif

				#if LOGGING_ON
				printf("send<--- err\n");
				#endif
				}
			else{
				iErr = ERR_IF;
				}
			}
		}

	#if SYS_ARCH_DO_LOG
	sInfo.pName		= "send done";
	sInfo.iMboxCount= sys_get_filled();
	TcpTesterPacket(&sInfo,pPtr,uLen,0);
	#endif


return iErr;
}

//*****************************************************************************
//*
//*		EthRecvThread
//*
//*****************************************************************************
//	Receive thread for incoming network packets
static void __cdecl EthRecvThread(void *pParam)
{
struct netif       *pInterface=(struct netif*)pParam;
unsigned char		aData[1600];
struct pbuf		   *pBuffer;
int					iSize;
unsigned			uRecv;
unsigned			uNum;
unsigned			uErr;
unsigned			uOk;




	#if SYS_ARCH_DO_LOG
	TteInfo	sInfo;
	#endif


	   uNum = pInterface->num;
	if(uNum>=MAX_INTERFACES)							// check interface number
		{
		return;
		}


	uRecv = 0;

	netif_set_up(pInterface);							// bring an interface up


	#if SYS_ARCH_DO_LOG
	memset(&sInfo,0,sizeof(sInfo));
	#endif


	#if PRINTING_ON
	if(!hPrintMux)hPrintMux=ThreadMutexInit();
	#endif


	for(;;)
		{												// get next packet from VNET
		   iSize = VnetRecv(aNetHandles[uNum],aData,sizeof(aData));
		if(iSize<=0)continue;

		uRecv++;

		#if PRINTING_ON
		ThreadMutexWait(hPrintMux);
		printf("recv---> %i\t%i\t%05u\n",iSize,uRecv,GetTickCount()%100000);
		ThreadMutexRelease(hPrintMux);
		#endif

		#if SYS_ARCH_DO_LOG
		sInfo.pName		= "recv";
		sInfo.iParam1	= iSize;
		sInfo.iParam2	= 0;
		sInfo.iMboxCount= sys_get_filled();
		TcpTesterPacket(&sInfo,aData,iSize,3);
		#endif

		uOk = 1;
		PIPE_STATS_INC(pipe.input);
														// allocate a buffer for the reseived packet
		   pBuffer = pbuf_alloc(PBUF_RAW,iSize,PBUF_RAM);
		if(pBuffer)
			{
			memcpy(pBuffer->payload,aData,iSize);
			uErr = pInterface->input(pBuffer,pInterface);

			if(uErr!=ERR_OK)
				{
				#if SYS_ARCH_DO_LOG
				sInfo.pName		 = "recv sleep";
				sInfo.iParam1	 = iSize;
				sInfo.iParam2	 = uErr;
				sInfo.iMboxCount = sys_get_filled();
				TcpTesterPacket(&sInfo,aData,iSize,1);
				#endif

				ThreadSleep(5);
														// put the packet to the LWIP stack
				uErr = pInterface->input(pBuffer,pInterface);

				if(uErr!=ERR_OK)
					{
					pbuf_free(pBuffer);					// dealloc the packet buffer at an error

					#if SYS_ARCH_DO_LOG
					SYS_LOG_DATA("recv error",1,aData,iSize,iSize,uErr,0);
					TcpTesterPacket(&sInfo,aData,iSize,1);
					TcpTesterError(0,0,0,"error input",0,0);
					#endif

					#if PRINTING_ON
					ThreadMutexWait(hPrintMux);
					printf("recv---> error input %i\t%i\n",iSize,uRecv);
					ThreadMutexRelease(hPrintMux);
					#endif

					PIPE_STATS_INC(pipe.inperr);
					uOk = 0;
					}
				}

			if(uErr==ERR_OK)							// packet successful handled
				{
				#if SYS_ARCH_DO_LOG
				sInfo.pName		 = "recv done";
				sInfo.iParam1	 = iSize;
				sInfo.iParam2	 = uErr;
				sInfo.iMboxCount = sys_get_filled();
				TcpTesterPacket(&sInfo,aData,iSize,1);
				#endif
				}
			}
		else{
			#if SYS_ARCH_DO_LOG
			sInfo.pName		 = "recv buffer error";
			sInfo.iParam1	 = iSize;
			sInfo.iMboxCount = sys_get_filled();
			TcpTesterError(0,0,0,"error buffer",0,0);
			#endif
			}
		}

	VnetClose(aNetHandles[uNum]);						// close VNET connection
	aNetHandles[uNum] = 0;

}

//*****************************************************************************
//*
//*		ArpTimer
//*
//*****************************************************************************
//	Callback function to clear expired ARB entries.
static void ArpTimer(void *pParaam)
{

	etharp_tmr();										// clear entries
	sys_timeout(ARP_TMR_INTERVAL,ArpTimer,NULL);		// register cb again

}

//*****************************************************************************
//*
//*		NetInterfaceInitCb
//*
//*****************************************************************************
//	Callback for netif_add(...), here the VNET interface will be activated
static err_t NetInterfaceInitCb(struct netif *pInterface)
{
unsigned	uNum;


	uNum = pInterface->num;

	LWIP_ASSERT("pInterface != NULL", (pInterface!=NULL));
	LWIP_ASSERT("pInterface->num   ", (uNum<MAX_INTERFACES));

														// open a VNET port
		aNetHandles[uNum] = VnetOpen(0,VNET_AUTOPORT,NULL,200,1,0);
	if(!aNetHandles[uNum])
		{
		#if PRINTING_ON
		ThreadMutexWait(hPrintMux);
		printf("\n\n>>>>>> can't open VNET port for interface %u <<<<<\n\n",uNum);
		ThreadMutexRelease(hPrintMux);
		#endif

		return ERR_IF;
		}

														// use MAC address from VNET port
	memcpy(pInterface->hwaddr,VnetMac(aNetHandles[uNum]),6);


	#if LWIP_NETIF_HOSTNAME
		pInterface->hostname =  IpGetHostname();		// initialize interface hostname
	if(*pInterface->hostname==0)IpSetHostname("lwip");	// see at SocketUtility.cpp
	#endif

	pInterface->name[0]		= 'e';
	pInterface->name[1]		= '0'+uNum;
	pInterface->flags	   |= NETIF_FLAG_ETHARP;
	pInterface->flags	   |= NETIF_FLAG_LINK_UP;
	pInterface->flags	   |= NETIF_FLAG_BROADCAST;
	pInterface->output		= etharp_output;
	pInterface->linkoutput	= EthSendProc;


	etharp_init();
	sys_timeout(ARP_TMR_INTERVAL,ArpTimer,NULL);		// start the arp timer

														// start the packet receive thread
	ThreadStartEx(EthRecvThread,0x8000,pInterface,THREAD_PRIORITY,"EthRecvThread");



return ERR_OK;
}

//*****************************************************************************
//*
//*		StringToIp
//*
//*****************************************************************************
//	Converts a string to a binary ip address
static int StringToIp(const char *pString,unsigned char *pAddr)
{
int		iPos;
int		iVal;


	for(iPos=0;iPos<4;iPos++)
		{
		if(iPos && *pString=='.')pString++;
		if(*pString<'0' || *pString>'9')return -1; 

		   iVal = atoi(pString);
		if(iVal&~0xFF)return -1;

		while(*pString>='0' && *pString<='9')
			{
			pString++;
			}
		
		pAddr[iPos] = iVal;
		}

	if(*pString || iPos<4)
		{
		return -1;
		}



return 0;
}

//*****************************************************************************
//*
//*		InitLwIp
//*
//*****************************************************************************
//	Initalizes all LWIP network interfaces
int InitLwIp(const char *pIpAddr,const char *pSubNet,const char *pGateway)
{
unsigned char	aGateway[4];
unsigned char	aIpAddr [4];
unsigned char	aSubNet [4];
struct ip_addr	sGateWay;
struct ip_addr	sNetMask;
struct ip_addr	sIpAddr;


	if(!pGateway || !*pGateway)pGateway="0.0.0.0";		// no gateway

	if(StringToIp(pIpAddr ,aIpAddr ))return -2;			 
	if(StringToIp(pSubNet ,aSubNet ))return -2;
	if(StringToIp(pGateway,aGateway))return -2;

	tcpip_init(NULL,NULL);								// create tcp_ip stack thread


	IP4_ADDR(&sIpAddr ,aIpAddr [0],aIpAddr [1],aIpAddr [2],aIpAddr [3]);
	IP4_ADDR(&sNetMask,aSubNet [0],aSubNet [1],aSubNet [2],aSubNet [3]);
	IP4_ADDR(&sGateWay,aGateway[0],aGateway[1],aGateway[2],aGateway[3]);

														// start the first interface
	if(!netif_add(aInterface+0,&sIpAddr,&sNetMask,&sGateWay,NULL,&NetInterfaceInitCb,&tcpip_input))
		{
		return -1;
		}


//	IP4_ADDR(&sIpAddr ,192,168, 98, 33);
//	IP4_ADDR(&sGateWay,192,168, 98,  1);
//	IP4_ADDR(&sNetMask,255,255,255,  0);
//
//														// start the next interface
//	if(!netif_add(aInterface+1,&sIpAddr,&sNetMask,&sGateWay,NULL,&NetInterfaceInitCb,&tcpip_input))
//		{
//		return -1;
//		}



return 0;
}
