/*
 * Copyright (c) 2001-2004 Swedish Institute of Computer Science.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without modification,
 * are permitted provided that the following conditions are met:
 *
 * 1. Redistributions of source code must retain the above copyright notice,
 *    this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright notice,
 *    this list of conditions and the following disclaimer in the documentation
 *    and/or other materials provided with the distribution.
 * 3. The name of the author may not be used to endorse or promote products
 *    derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT
 * SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT
 * OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING
 * IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY
 * OF SUCH DAMAGE.
 *
 * This file is part of the lwIP TCP/IP stack.
 *
 * Author: Adam Dunkels <adam@sics.se>
 *
 */
#ifndef __LWIP_STATS_H__
#define __LWIP_STATS_H__

#include "lwip/opt.h"

#include "lwip/mem.h"
#include "lwip/memp.h"

#ifdef __cplusplus
extern "C" {
#endif

#if LWIP_STATS

// REM AZ Erweiterung
#define LWIP_STATS_PRINT(a)	if(*pos<size-64)*pos+=sprintf a
#define LWIP_STATS_PARAMS		char *puffer,int *pos,int size
#define LWIP_STATS_PARAMS_CALL	puffer ,pos,size
#define LWIP_STATS_PARAMS_INIT  puffer+*pos,"\n"
#define LWIP_STATS_PARAMS_PRE   puffer+*pos,
#define LWIP_STATS_PARAMS_MIDD  "\n\t"
#define LWIP_STATS_PARAMS_POST



/* LWIP_STATS_PARAMS are the excented parameters for the stats_display_??? functions
   i.e. :    LWIP_STATS_PARAMS         char *puffer,int *size
   LWIP_STATS_PARAMS_INIT are the first parameters for a stats_display_??? output call of the first line
   i.e. :    LWIP_STATS_PARAMS_INIT    puffer+*size,"\n"
   LWIP_STATS_PARAMS_PRE are the first parameters for a stats_display_??? output call
   i.e. :    LWIP_STATS_PARAMS_PRE     puffer+*size,
   LWIP_STATS_PARAMS_POST are the middle parameters for a stats_display_??? after the format string
   i.e. :    LWIP_STATS_PARAMS_MIDD    ," %s\n\t"
   LWIP_STATS_PARAMS_POST are the first parameters for a stats_display_??? output call
   i.e. :    LWIP_STATS_PARAMS_POST    ," X"
   LWIP_STATS_PARAMS_CALL are the excented parameters for a stats_display_??? function call
   i.e. :    LWIP_STATS_PARAMS_CALL    puffer,size
   LWIP_STATS_PPRINT  is the used output function for statisics
   i.e. :    LWIP_STATS_PRINT(a)		*size += sprintf a

   |.... PRINT......|...PRE/INIT..|......format......|..MIDD...|...arument...|.POST.|
   *size += sprintf( puffer+*size, "sem.used: %"U32_F " %s\n\t" sys->sem.used ," X"	 );

*/

#ifndef	LWIP_STATS_PARAMS
#define LWIP_STATS_PARAMS
#define LWIP_STATS_PARAMS_EX
#define LWIP_STATS_PARAMS_CL
#else
#define LWIP_STATS_PARAMS_EX	,LWIP_STATS_PARAMS
#define LWIP_STATS_PARAMS_CL	,LWIP_STATS_PARAMS_CALL
#endif

#ifndef	LWIP_STATS_PARAMS_INIT
#define LWIP_STATS_PARAMS_INIT "\n"
#endif

#ifndef	LWIP_STATS_PARAMS_PRE
#define LWIP_STATS_PARAMS_PRE
#endif

#ifndef	LWIP_STATS_PARAMS_MIDD
#define LWIP_STATS_PARAMS_MIDD	"\n\t"
#endif

#ifndef	LWIP_STATS_PARAMS_POST
#define LWIP_STATS_PARAMS_POST
#endif




#ifndef LWIP_STATS_LARGE
#define LWIP_STATS_LARGE 1
#endif

#if LWIP_STATS_LARGE
#define STAT_SIGNED      s32_t
#define STAT_COUNTER     u32_t
#define STAT_COUNTER_F   U32_F
#else
#define STAT_SIGNED      s16_t
#define STAT_COUNTER     u16_t
#define STAT_COUNTER_F   U16_F
#endif



struct stats_proto {
  STAT_COUNTER xmit;             /* Transmitted packets. */
  STAT_COUNTER recv;             /* Received packets. */
  STAT_COUNTER fw;               /* Forwarded packets. */
  STAT_COUNTER drop;             /* Dropped packets. */
  STAT_COUNTER chkerr;           /* Checksum error. */
  STAT_COUNTER lenerr;           /* Invalid length error. */
  STAT_COUNTER memerr;           /* Out of memory error. */
  STAT_COUNTER rterr;            /* Routing error. */
  STAT_COUNTER proterr;          /* Protocol error. */
  STAT_COUNTER opterr;           /* Error in options. */
  STAT_COUNTER err;              /* Misc error. */
  STAT_COUNTER cachehit;
};

struct stats_igmp {
  STAT_COUNTER xmit;             /* Transmitted packets. */
  STAT_COUNTER recv;             /* Received packets. */
  STAT_COUNTER drop;             /* Dropped packets. */
  STAT_COUNTER chkerr;           /* Checksum error. */
  STAT_COUNTER lenerr;           /* Invalid length error. */
  STAT_COUNTER memerr;           /* Out of memory error. */
  STAT_COUNTER proterr;          /* Protocol error. */
  STAT_COUNTER rx_v1;            /* Received v1 frames. */
  STAT_COUNTER rx_group;         /* Received group-specific queries. */
  STAT_COUNTER rx_general;       /* Received general queries. */
  STAT_COUNTER rx_report;        /* Received reports. */
  STAT_COUNTER tx_join;          /* Sent joins. */
  STAT_COUNTER tx_leave;         /* Sent leaves. */
  STAT_COUNTER tx_report;        /* Sent reports. */
};

struct stats_mem {
#ifdef LWIP_DEBUG
  const char *name;
#endif /* LWIP_DEBUG */
  mem_size_t avail;
  mem_size_t used;
  mem_size_t max;
  STAT_COUNTER err;
  STAT_COUNTER illegal;
};

struct stats_syselem {
  STAT_COUNTER used;
  STAT_COUNTER max;
  STAT_COUNTER err;
};

struct stats_pipe {		// REM AZ Erweiterung
  STAT_COUNTER input;
  STAT_COUNTER inperr;
  STAT_COUNTER output;
  STAT_COUNTER outerr;
  STAT_SIGNED  mboxnow;
  STAT_SIGNED  mboxmax;
  STAT_COUNTER mboxerr;
};

struct stats_open {		// REM AZ Erweiterung
  unsigned short port[OPEN_STATS_MAX];
  STAT_COUNTER   open[OPEN_STATS_MAX];
  STAT_COUNTER   close[OPEN_STATS_MAX];
};

struct stats_sys {
  struct stats_syselem sem;
  struct stats_syselem mutex;
  struct stats_syselem mbox;
};

struct stats_ {
#if LINK_STATS
  struct stats_proto link;
#endif
#if ETHARP_STATS
  struct stats_proto etharp;
#endif
#if IPFRAG_STATS
  struct stats_proto ip_frag;
#endif
#if IP_STATS
  struct stats_proto ip;
#endif
#if ICMP_STATS
  struct stats_proto icmp;
#endif
#if IGMP_STATS
  struct stats_igmp igmp;
#endif
#if UDP_STATS
  struct stats_proto udp;
#endif
#if TCP_STATS
  struct stats_proto tcp;
#endif
#if MEM_STATS
  struct stats_mem mem;
#endif
#if MEMP_STATS
  struct stats_mem memp[MEMP_MAX];
#endif
#if SYS_STATS
  struct stats_sys sys;
#endif
#if PIPE_STATS
  struct stats_pipe pipe;	// REM AZ Erweiterung
#endif
#if OPEN_STATS
  struct stats_open open;	// REM AZ Erweiterung
#endif
};

extern struct stats_ lwip_stats;

void stats_init(void);

#define STATS_INC(x) ++lwip_stats.x
#define STATS_DEC(x) --lwip_stats.x
#define STATS_INC_USED(x, y) do { lwip_stats.x.used += y; \
                                if (lwip_stats.x.max < lwip_stats.x.used) { \
                                    lwip_stats.x.max = lwip_stats.x.used; \
                                } \
                             } while(0)
#else /* LWIP_STATS */
#define stats_init()
#define STATS_INC(x)
#define STATS_DEC(x)
#define STATS_INC_USED(x)
#endif /* LWIP_STATS */

#if TCP_STATS
#define TCP_STATS_INC(x) STATS_INC(x)
#define TCP_STATS_DISPLAY() stats_display_proto(&lwip_stats.tcp, "TCP" LWIP_STATS_PARAMS_CL)
#else
#define TCP_STATS_INC(x)
#define TCP_STATS_DISPLAY()
#endif

#if UDP_STATS
#define UDP_STATS_INC(x) STATS_INC(x)
#define UDP_STATS_DISPLAY() stats_display_proto(&lwip_stats.udp, "UDP" LWIP_STATS_PARAMS_CL)
#else
#define UDP_STATS_INC(x)
#define UDP_STATS_DISPLAY()
#endif

#if ICMP_STATS
#define ICMP_STATS_INC(x) STATS_INC(x)
#define ICMP_STATS_DISPLAY() stats_display_proto(&lwip_stats.icmp, "ICMP" LWIP_STATS_PARAMS_CL)
#else
#define ICMP_STATS_INC(x)
#define ICMP_STATS_DISPLAY()
#endif

#if PIPE_STATS
#define PIPE_STATS_DEC(x) STATS_DEC(x)
#define PIPE_STATS_INC(x) STATS_INC(x)
#define PIPE_STATS_MAX(x,m)  if((short)(lwip_stats.x)>(short)(lwip_stats.m))lwip_stats.m=lwip_stats.x
#define PIPE_STATS_DISPLAY() stats_display_pipe(&lwip_stats.pipe LWIP_STATS_PARAMS_CL)
#else
#define PIPE_STATS_DEC(x)
#define PIPE_STATS_INC(x)
#define PIPE_STATS_MAX(x,m)
#define PIPE_STATS_DISPLAY()
#endif

#if OPEN_STATS	
extern  void stats_open_change(struct stats_open *open,unsigned short port,int mode);					  
#define OPEN_STATS_DEC(err,x)	if(!(err))stats_open_change(&lwip_stats.open,x,0)
#define OPEN_STATS_INC(err,x)	if(!(err))stats_open_change(&lwip_stats.open,x,1)
#define OPEN_STATS_DISPLAY()	stats_display_open(&lwip_stats.open LWIP_STATS_PARAMS_CL)
#else
#define OPEN_STATS_DEC(x)
#define OPEN_STATS_INC(x)
#define OPEN_STATS_DISPLAY()
#endif

#if CONN_STATS
#define CONN_STATS_DISPLAY() stats_display_conn(tcp_pcb_lists,4 LWIP_STATS_PARAMS_CL)
#else
#define CONN_STATS_DISPLAY()
#endif

#if IGMP_STATS
#define IGMP_STATS_INC(x) STATS_INC(x)
#define IGMP_STATS_DISPLAY() stats_display_igmp(&lwip_stats.igmp LWIP_STATS_PARAMS_CL)
#else
#define IGMP_STATS_INC(x)
#define IGMP_STATS_DISPLAY()
#endif

#if IP_STATS
#define IP_STATS_INC(x) STATS_INC(x)
#define IP_STATS_DISPLAY() stats_display_proto(&lwip_stats.ip, "IP" LWIP_STATS_PARAMS_CL)
#else
#define IP_STATS_INC(x)
#define IP_STATS_DISPLAY()
#endif

#if IPFRAG_STATS
#define IPFRAG_STATS_INC(x) STATS_INC(x)
#define IPFRAG_STATS_DISPLAY() stats_display_proto(&lwip_stats.ip_frag, "IP_FRAG" LWIP_STATS_PARAMS_CL)
#else
#define IPFRAG_STATS_INC(x)
#define IPFRAG_STATS_DISPLAY()
#endif

#if ETHARP_STATS
#define ETHARP_STATS_INC(x) STATS_INC(x)
#define ETHARP_STATS_DISPLAY() stats_display_proto(&lwip_stats.etharp, "ETHARP" LWIP_STATS_PARAMS_CL)
#else
#define ETHARP_STATS_INC(x)
#define ETHARP_STATS_DISPLAY()
#endif

#if LINK_STATS
#define LINK_STATS_INC(x) STATS_INC(x)
#define LINK_STATS_DISPLAY() stats_display_proto(&lwip_stats.link, "LINK" LWIP_STATS_PARAMS_CL)
#else
#define LINK_STATS_INC(x)
#define LINK_STATS_DISPLAY()
#endif

#if MEM_STATS
#define MEM_STATS_AVAIL(x, y) lwip_stats.mem.x = y
#define MEM_STATS_INC(x) STATS_INC(mem.x)
#define MEM_STATS_INC_USED(x, y) STATS_INC_USED(mem, y)
#define MEM_STATS_DEC_USED(x, y) lwip_stats.mem.x -= y
#define MEM_STATS_DISPLAY() stats_display_mem(&lwip_stats.mem, "HEAP" LWIP_STATS_PARAMS_CL)
#else
#define MEM_STATS_AVAIL(x, y)
#define MEM_STATS_INC(x)
#define MEM_STATS_INC_USED(x, y)
#define MEM_STATS_DEC_USED(x, y)
#define MEM_STATS_DISPLAY()
#endif

#if MEMP_STATS
#define MEMP_STATS_AVAIL(x, i, y) lwip_stats.memp[i].x = y
#define MEMP_STATS_INC(x, i) STATS_INC(memp[i].x)
#define MEMP_STATS_DEC(x, i) STATS_DEC(memp[i].x)
#define MEMP_STATS_INC_USED(x, i) STATS_INC_USED(memp[i], 1)
#define MEMP_STATS_DISPLAY(i) stats_display_memp(&lwip_stats.memp[i], i LWIP_STATS_PARAMS_CL)
#else
#define MEMP_STATS_AVAIL(x, i, y)
#define MEMP_STATS_INC(x, i)
#define MEMP_STATS_DEC(x, i)
#define MEMP_STATS_INC_USED(x, i)
#define MEMP_STATS_DISPLAY(i)
#endif

#if SYS_STATS
#define SYS_STATS_INC(x) STATS_INC(sys.x)
#define SYS_STATS_DEC(x) STATS_DEC(sys.x)
#define SYS_STATS_INC_USED(x) STATS_INC_USED(sys.x, 1)
#define SYS_STATS_DISPLAY() stats_display_sys(&lwip_stats.sys LWIP_STATS_PARAMS_CL)
#else
#define SYS_STATS_INC(x)
#define SYS_STATS_DEC(x)
#define SYS_STATS_INC_USED(x)
#define SYS_STATS_DISPLAY()
#endif

/* Display of statistics */
#if LWIP_STATS_DISPLAY
void stats_display(LWIP_STATS_PARAMS);
void stats_display_proto(struct stats_proto *proto, const char *name LWIP_STATS_PARAMS_EX);
void stats_display_igmp (struct stats_igmp  *igmp                    LWIP_STATS_PARAMS_EX);
void stats_display_mem  (struct stats_mem   *mem, const char *name   LWIP_STATS_PARAMS_EX);
void stats_display_memp (struct stats_mem   *mem, int index          LWIP_STATS_PARAMS_EX);
void stats_display_sys  (struct stats_sys   *sys                     LWIP_STATS_PARAMS_EX);
void stats_display_pipe (struct stats_pipe  *pipe                    LWIP_STATS_PARAMS_EX);
#else /* LWIP_STATS_DISPLAY */
#define stats_display()
#define stats_display_proto(proto, name)
#define stats_display_igmp(igmp)
#define stats_display_mem(mem, name)
#define stats_display_memp(mem, index)
#define stats_display_sys(sys)
#define stats_display_pipe(pipe)
#endif /* LWIP_STATS_DISPLAY */

#ifdef __cplusplus
}
#endif

#endif /* __LWIP_STATS_H__ */
