//*****************************************************************************
//*
//*
//*		sys.h
//*
//*
//*****************************************************************************
#ifndef		__LWIP_SYS_H__
#define		__LWIP_SYS_H__

#include	"lwip/opt.h"

#ifdef __cplusplus
extern "C" {
#endif

#if NO_SYS

// For a totally minimal and standalone system, we provide null definitions of the sys_ functions.
typedef		u8_t		sys_sem_t;
typedef 	u8_t		sys_mutex_t;
typedef		u8_t		sys_mbox_t;

#define		sys_init()
#define 	sys_timeout(m,h,a)
#define 	sys_untimeout(m,a)
#define 	sys_sem_new(s,c)  ERR_OK
#define 	sys_sem_signal(s)
#define 	sys_sem_wait(s)
#define 	sys_sem_wait_ex(s,t)
#define 	sys_sem_free(s)
#define 	sys_mutex_new(mu) ERR_OK
#define 	sys_mutex_lock(mu)
#define 	sys_mutex_unlock(mu)
#define 	sys_mutex_free(mu)
#define 	sys_mbox_new(s,d) ERR_OK
#define 	sys_mbox_fetch(m,d)
#define 	sys_mbox_tryfetch(m,d)
#define 	sys_mbox_post(m,d)
#define 	sys_mbox_trypost(m,d)
#define 	sys_mbox_free(m)
#define 	sys_thread_new(n,t,a,s,p)

#ifndef 	SYS_MEM_POOL_MAP
#define 	SYS_MEM_POOL_MAP(memp)		// REM AZ Erweiterung
#endif 	

#ifndef		SYS_MSG_EXTENSION
#define 	SYS_MSG_EXTENSION			// REM AZ Erweiterung
#endif 	

#else

//*****************************************************************************
//*
//*		REM AZ	implement the own system functions
//*
//*****************************************************************************

#include "arch/sys_arch.h"
#include "lwip/iptimers.h"
#include "lwip/err.h"


#define					SYS_MBOX_EMPTY						SYS_ARCH_TIMEOUT
#ifndef					SYS_MSG_EXTENSION
#define 				SYS_MSG_EXTENSION
#endif

#define 				SYS_MEM_POOL_MAP(memp)				sys_arch_pool_map(memp);

// Thread functions
void		 			sys_init(void);
#define 				sys_thread_new(n,f,a,s,p)			sys_arch_thread_new(n,f,a,s,p)

// Time functions
#define 				sys_msleep(ms)						sys_arch_sleep(ms)
#define 				sys_jiffies()						sys_arch_jiffies()
#define 				sys_now()							sys_arch_now()

// Mutex functions
#define 				sys_mutex_t                 		sys_sem_t
#define 				sys_mutex_new(mutex)          		sys_sem_new        (mutex,1)
#define 				sys_mutex_lock(mutex)         		sys_sem_wait       (mutex)
#define 				sys_mutex_unlock(mutex)       		sys_sem_signal     (mutex)
#define 				sys_mutex_free(mutex)         		sys_sem_free       (mutex)
#define 				sys_mutex_valid(mutex)        		sys_sem_valid      (mutex)
#define 				sys_mutex_set_invalid(mutex)  		sys_sem_set_invalid(mutex)

// Semaphore functions.
#define					sys_sem_signal(sem)					sys_arch_sem_signal     (sem)
#define					sys_sem_valid(sem)					sys_arch_sem_valid      (sem)
#define					sys_sem_set_invalid(sem)			sys_arch_sem_set_invalid(sem)
#define					sys_sem_new(sem,count)				sys_arch_sem_new        (sem,count)
#define 				sys_sem_wait(sem)					sys_arch_sem_wait       (sem,0)
#define					sys_sem_wait_ex(sem,t)				sys_arch_sem_wait       (sem,t)
#define					sys_sem_free(sem)					sys_arch_sem_free	   (*sem)

// Mailbox functions.
#define 				sys_mbox_new(mbox,size)				sys_arch_mbox_new        (mbox,size)
#define 				sys_mbox_free(mbox)					sys_arch_mbox_free      (*mbox)
#define 				sys_mbox_valid(mbox)				sys_arch_mbox_valid      (mbox)
#define					sys_mbox_set_invalid(mbox)			sys_arch_mbox_set_invalid(mbox)
#define 				sys_mbox_fetch(mbox,msg)			sys_arch_mbox_fetch      (mbox,msg,0)
#define					sys_mbox_tryfetch(mbox,msg)			sys_arch_mbox_tryfetch   (mbox,msg)
#define					sys_mbox_trypost(mbox,msg)			sys_arch_mbox_trypost    (mbox,msg)
#define					sys_mbox_post(mbox,msg)				sys_arch_mbox_post       (mbox,msg)

// Priority Mailbox functions.
#define 				sys_pipe_valid(pipe)				sys_arch_pipe_valid(pipe)
#define					sys_pipe_set_invalid(pipe)			sys_arch_pipe_set_invalid(pipe)
#define					sys_pipe_free(pipe)					sys_arch_pipe_free(pipe)
#define					sys_pipe_new(pipe,size)				sys_arch_pipe_new(pipe,size)
#define					sys_pipe_fetch(pipe,msg,timeo)		sys_arch_pipe_fetch(pipe,msg,timeo)
#define					sys_pipe_tryfetch(pipe,msg)			sys_arch_pipe_tryfetch(pipe,msg)
#define					sys_pipe_trypost(pipe,msg)			sys_arch_pipe_trypost(pipe,msg)
#define					sys_pipe_post(pipe,msg)				sys_arch_pipe_post(pipe,msg)


//*****************************************************************************
#endif

#ifndef		SYS_ARCH_PROTECT


#if			SYS_LIGHTWEIGHT_PROT
#define		SYS_ARCH_DECL_PROTECT(lev)	sys_prot_t lev
#define		SYS_ARCH_PROTECT(lev)		lev = sys_arch_protect()
#define		SYS_ARCH_UNPROTECT(lev)		sys_arch_unprotect(lev)

#else

#define		SYS_ARCH_DECL_PROTECT(lev)
#define 	SYS_ARCH_UNPROTECT(lev)
#define 	SYS_ARCH_PROTECT(lev)

#endif

#endif


// Macros to set/get and increase/decrease variables in a thread-safe way.
// Use these for accessing variable that are used from more than one thread.


#ifndef SYS_ARCH_INC
#define SYS_ARCH_INC(var, val) do {									\
                                SYS_ARCH_DECL_PROTECT(old_level);	\
                                SYS_ARCH_PROTECT(old_level);		\
                                var += val;							\
                                SYS_ARCH_UNPROTECT(old_level);		\
								} while(0)
#endif

#ifndef SYS_ARCH_DEC
#define SYS_ARCH_DEC(var, val) do {									\
                                SYS_ARCH_DECL_PROTECT(old_level);	\
                                SYS_ARCH_PROTECT(old_level);		\
                                var -= val;							\
                                SYS_ARCH_UNPROTECT(old_level);		\
								} while(0)
#endif

#ifndef SYS_ARCH_GET
#define SYS_ARCH_GET(var, ret) do {									\
                                SYS_ARCH_DECL_PROTECT(old_level);	\
                                SYS_ARCH_PROTECT(old_level);		\
                                ret = var;							\
                                SYS_ARCH_UNPROTECT(old_level);		\
								} while(0)
#endif

#ifndef SYS_ARCH_SET
#define SYS_ARCH_SET(var, val) do { \
                                SYS_ARCH_DECL_PROTECT(old_level);	\
                                SYS_ARCH_PROTECT(old_level);		\
                                var = val;							\
                                SYS_ARCH_UNPROTECT(old_level);		\
								} while(0)
#endif


#ifdef __cplusplus
}
#endif



#endif
